searchNodes=[{"ref":"ExDimensions.html","title":"ExDimensions","type":"module","doc":"ex_dimensions is a library for providing safe unit math and conversions. Units are tracked inside a struct that is labeled with phantom types for the units of the number, and transparent math is provided by macros. To get started, first any simple numeric must be &quot;lifted&quot; into a unit aware number. The easiest way is using the helper functions for the type of quantity you want to work with: iex&gt; q = ExDimensions.Spatial.inches(6) iex&gt; IO.puts(q) &quot;6 in&quot; iex&gt; IO.inspect(q) %ExDimensions.Quantity{value: 6, units: [ExDimensions.Spatial.Inches], denom: []} Quantities can be constructed manually but as they must obey certain rules, this is not recommended. Quantity arithmetic can be done transparently by using the ExDimensions.Math macros: iex&gt; defmodule Foo do iex&gt; use ExDimensions.Math iex&gt; def do_some_math() do iex&gt; q1 = ExDimensions.Spatial.inches(6) iex&gt; q2 = ExDimensions.Spatial.inches(3) iex&gt; q1 + q2 iex&gt; end iex&gt; end iex&gt; IO.puts(Foo.do_some_math()) &quot;9 in&quot; Incorrect units supplied to math functions will result in ArithmeticErrors. There is also a unit conversion framework available. See ExDimensions.Conversions for an example of how to use it. Finally, a custom Ecto type is provided for those who want to transparently persist numbers with units. See ExDimensions.Ecto.UnitField for an explanation."},{"ref":"ExDimensions.Area.html","title":"ExDimensions.Area","type":"module","doc":"Base units for area measurements in metric and imperial."},{"ref":"ExDimensions.Area.html#km_squared/1","title":"ExDimensions.Area.km_squared/1","type":"function","doc":""},{"ref":"ExDimensions.Area.html#m_squared/1","title":"ExDimensions.Area.m_squared/1","type":"function","doc":""},{"ref":"ExDimensions.Area.html#micros_squared/1","title":"ExDimensions.Area.micros_squared/1","type":"function","doc":""},{"ref":"ExDimensions.Area.html#mm_squared/1","title":"ExDimensions.Area.mm_squared/1","type":"function","doc":""},{"ref":"ExDimensions.Area.html#nm_squared/1","title":"ExDimensions.Area.nm_squared/1","type":"function","doc":""},{"ref":"ExDimensions.Area.html#square_feet/1","title":"ExDimensions.Area.square_feet/1","type":"function","doc":""},{"ref":"ExDimensions.Area.html#square_inches/1","title":"ExDimensions.Area.square_inches/1","type":"function","doc":""},{"ref":"ExDimensions.Area.html#square_miles/1","title":"ExDimensions.Area.square_miles/1","type":"function","doc":""},{"ref":"ExDimensions.Area.html#square_yards/1","title":"ExDimensions.Area.square_yards/1","type":"function","doc":""},{"ref":"ExDimensions.Conversions.html","title":"ExDimensions.Conversions","type":"module","doc":"A module containing the DSL definitions for unit conversions"},{"ref":"ExDimensions.Conversions.html#convert/3","title":"ExDimensions.Conversions.convert/3","type":"function","doc":""},{"ref":"ExDimensions.Conversions.html#~>/2","title":"ExDimensions.Conversions.~>/2","type":"function","doc":"DSL function for converting a quantity of a given value to some other unit, if there is an available conversion. If so, the new quantity is returned. If not, then an error tuple is returned. This module must be imported into the current scope for this function to be available. Examples iex&gt; import ExDimensions.Conversions iex&gt; IO.puts(ExDimensions.Spatial.feet(1) ~&gt; ExDimensions.Spatial.Inches) &quot;12 in&quot;"},{"ref":"ExDimensions.Ecto.UnitField.html","title":"ExDimensions.Ecto.UnitField","type":"module","doc":"A custom Ecto type for integration into Ecto based projects. The column is defined as a :map Ecto type with 2 keys: units: string representation of the units value: numerical value of the quantity For Postgres databases, that will map to a jsonb column. Other database implementations will vary."},{"ref":"ExDimensions.Ecto.UnitField.html#cast/1","title":"ExDimensions.Ecto.UnitField.cast/1","type":"function","doc":"Casts the given input to the custom type. This callback is called on external input and can return any type, as long as the dump/1 function is able to convert the returned value into an Ecto native type. There are two situations where this callback is called: When casting values by Ecto.Changeset When passing arguments to Ecto.Query When returning {:error, keyword()}, the returned keyword list will be preserved in the changeset errors, similar to Changeset.add_error/4. Passing a :message key, will override the default message. It is not possible to override the :type key. For {:array, CustomType} or {:map, CustomType} the returned keyword list will be erased and the default error will be shown. Callback implementation for Ecto.Type.cast/1."},{"ref":"ExDimensions.Ecto.UnitField.html#dump/1","title":"ExDimensions.Ecto.UnitField.dump/1","type":"function","doc":"Dumps the given term into an Ecto native type. This callback is called with any term that was stored in the struct and it needs to validate them and convert it to an Ecto native type. Callback implementation for Ecto.Type.dump/1."},{"ref":"ExDimensions.Ecto.UnitField.html#embed_as/1","title":"ExDimensions.Ecto.UnitField.embed_as/1","type":"function","doc":"Dictates how the type should be treated inside embeds. By default, the type is sent as itself, without calling dumping to keep the higher level representation. But it can be set to :dump to it is dumped before encoded. Callback implementation for Ecto.Type.embed_as/1."},{"ref":"ExDimensions.Ecto.UnitField.html#equal?/2","title":"ExDimensions.Ecto.UnitField.equal?/2","type":"function","doc":"Checks if two terms are semantically equal. Callback implementation for Ecto.Type.equal?/2."},{"ref":"ExDimensions.Ecto.UnitField.html#load/1","title":"ExDimensions.Ecto.UnitField.load/1","type":"function","doc":"Loads the given term into a custom type. This callback is called when loading data from the database and receives an Ecto native type. It can return any type, as long as the dump/1 function is able to convert the returned value back into an Ecto native type. Callback implementation for Ecto.Type.load/1."},{"ref":"ExDimensions.Ecto.UnitField.html#parse/1","title":"ExDimensions.Ecto.UnitField.parse/1","type":"function","doc":""},{"ref":"ExDimensions.Ecto.UnitField.html#type/0","title":"ExDimensions.Ecto.UnitField.type/0","type":"function","doc":"Returns the underlying schema type for the custom type. For example, if you want to provide your own date structures, the type function should return :date. Note this function is not required to return Ecto primitive types, the type is only required to be known by the adapter. Callback implementation for Ecto.Type.type/0."},{"ref":"ExDimensions.Mass.html","title":"ExDimensions.Mass","type":"module","doc":"Base units for mass measurements in metric and imperial."},{"ref":"ExDimensions.Mass.html#grams/1","title":"ExDimensions.Mass.grams/1","type":"function","doc":""},{"ref":"ExDimensions.Mass.html#kilograms/1","title":"ExDimensions.Mass.kilograms/1","type":"function","doc":""},{"ref":"ExDimensions.Mass.html#ounces/1","title":"ExDimensions.Mass.ounces/1","type":"function","doc":""},{"ref":"ExDimensions.Mass.html#pounds/1","title":"ExDimensions.Mass.pounds/1","type":"function","doc":""},{"ref":"ExDimensions.Math.html","title":"ExDimensions.Math","type":"module","doc":"This module contains the required macros to perform math on unit quantities. It must be used in the current scope for the math functions to work properly. Math on standard values will still work as expected if this module is used. Using these math functions will ensure proper dimensional analysis is performed on quantities. The following rules are enforced: Quantities with different units may not be added or subtracted Quantities may not be added or subtracted with plain scalar numbers Quantities may be multiplied and divided with scalars Quantities with different units that are multiplied and divided will have their units changed as needed Any math operations that violate these rules will result in an ArithmeticError that will bubble up to the caller."},{"ref":"ExDimensions.Quantity.html","title":"ExDimensions.Quantity","type":"module","doc":"The core quantity type for units. It maintains bookkeeping around unit manipulations as math and conversion operations are applied. It is not recommended to create this struct directly. Rather, helpers such as ExDimensions.Spatial.inches(5) should be used to initialize quantities, and manipulations of the value should take place using the macro defined in ExDimensions.UnitMath."},{"ref":"ExDimensions.Spatial.html","title":"ExDimensions.Spatial","type":"module","doc":"Base units for spatial measurements in metric and imperial. See ExDimensions.Spatial.Area and ExDimensions.Spatial.Volume for helpers that can create quantities in square and cubed units."},{"ref":"ExDimensions.Spatial.html#feet/1","title":"ExDimensions.Spatial.feet/1","type":"function","doc":""},{"ref":"ExDimensions.Spatial.html#inches/1","title":"ExDimensions.Spatial.inches/1","type":"function","doc":""},{"ref":"ExDimensions.Spatial.html#kilometers/1","title":"ExDimensions.Spatial.kilometers/1","type":"function","doc":""},{"ref":"ExDimensions.Spatial.html#meters/1","title":"ExDimensions.Spatial.meters/1","type":"function","doc":""},{"ref":"ExDimensions.Spatial.html#micrometers/1","title":"ExDimensions.Spatial.micrometers/1","type":"function","doc":""},{"ref":"ExDimensions.Spatial.html#miles/1","title":"ExDimensions.Spatial.miles/1","type":"function","doc":""},{"ref":"ExDimensions.Spatial.html#millimeters/1","title":"ExDimensions.Spatial.millimeters/1","type":"function","doc":""},{"ref":"ExDimensions.Spatial.html#nanometers/1","title":"ExDimensions.Spatial.nanometers/1","type":"function","doc":""},{"ref":"ExDimensions.Spatial.html#yards/1","title":"ExDimensions.Spatial.yards/1","type":"function","doc":""},{"ref":"ExDimensions.Temperature.html","title":"ExDimensions.Temperature","type":"module","doc":"Base units for temperature measurements."},{"ref":"ExDimensions.Temperature.html#celsius/1","title":"ExDimensions.Temperature.celsius/1","type":"function","doc":""},{"ref":"ExDimensions.Temperature.html#fahrenheit/1","title":"ExDimensions.Temperature.fahrenheit/1","type":"function","doc":""},{"ref":"ExDimensions.Temperature.html#kelvin/1","title":"ExDimensions.Temperature.kelvin/1","type":"function","doc":""},{"ref":"ExDimensions.Unit.html","title":"ExDimensions.Unit","type":"behaviour","doc":"Base behavior for a unit type."},{"ref":"ExDimensions.Unit.html#c:abbr/0","title":"ExDimensions.Unit.abbr/0","type":"callback","doc":""},{"ref":"ExDimensions.Volume.html","title":"ExDimensions.Volume","type":"module","doc":"Base units for volume measurements in metric and imperial."},{"ref":"ExDimensions.Volume.html#cubic_feet/1","title":"ExDimensions.Volume.cubic_feet/1","type":"function","doc":""},{"ref":"ExDimensions.Volume.html#cubic_inches/1","title":"ExDimensions.Volume.cubic_inches/1","type":"function","doc":""},{"ref":"ExDimensions.Volume.html#m_cubed/1","title":"ExDimensions.Volume.m_cubed/1","type":"function","doc":""},{"ref":"ExDimensions.Volume.html#mm_cubed/1","title":"ExDimensions.Volume.mm_cubed/1","type":"function","doc":""}]